/*
   model/face.cc
   This file is part of the Osirion project and is distributed under
   the terms of the GNU General Public License version 2
*/

#include <string>

#include "model/face.h"

namespace model
{

using math::Vector3f;
/*
 *  all points p(x, y, z) on the plane satisfy the general equation
 *  x*a() + y*b() + z*c() + d() = 0
 */

Face::Face(Vector3f const & point0, Vector3f const &point1, Vector3f const &point2)
{
	face_detail = false;
	face_surface_flags = 0;
	face_material = 0;

	face_point[0] = point0;
	face_point[1] = point1;
	face_point[2] = point2;

	face_normal = crossproduct((face_point[1] - face_point[0]) , (face_point[2] - face_point[0]));
	pd = -1 * (face_normal.x() * face_point[0].x() + face_normal.y() * face_point[0].y() + face_normal.z() * face_point[0].z());
}

Face::Face(Face const & other)
{
	for (size_t i = 0; i < 3; i++)
		this->face_point[i] = other.face_point[i];

	face_normal = crossproduct((face_point[1] - face_point[0]) , (face_point[2] - face_point[0]));
	pd = -1 * (face_normal.x() * face_point[0].x() + face_normal.y() * face_point[0].y() + face_normal.z() * face_point[0].z());
}

}
