/*
   model/tags.cc
   This file is part of the Osirion project and is distributed under
   the terms of the GNU General Public License version 2
*/

#include "model/model.h"
#include "model/tags.h"

namespace model
{

/* ---- class Tag -------------------------------------------------- */

Tag::Tag() : tag_location()
{
}

Tag::Tag(const Tag& other) : tag_location(other.location())
{
}

Tag::~Tag()
{
}

/* ---- class Light ------------------------------------------------ */

Light::Light() :
	Tag(),
	light_color(1.0f, 1.0f, 1.0f)
{
	light_entity = false;
	light_entity_second = false;
	light_engine = false;
	light_strobe = false;
	light_has_color = false;

	light_radius = 100.0f * SCALE;
	light_frequency = 1.0f;
	light_offset = 0.0f;
	light_time = 0.5f;

	light_flare = 0;

	light_texture = 0;
}

Light::Light(const Light& other) : 
	Tag(other),
	light_color(other.color())
{
	light_entity = other.entity();
	light_entity_second = other.entity_second();
	light_engine = other.engine();
	light_strobe = other.strobe();
	light_has_color = other.has_color();

	light_radius = other.radius();
	light_frequency = other.frequency();
	light_offset = other.offset();
	light_time = other.time();

	light_flare = other.flare();

	light_texture = other.texture();
}

Light::~Light()
{
}

/* ---- class Flare ------------------------------------------------ */

Flare::Flare() : 
	Light(),
	flare_axis()
{
	flare_cull = CullBack;
}

Flare::Flare(const Flare& other) : 
	Light(other),
	flare_axis(other.axis())
{
	flare_cull = other.cull();
}

Flare::~Flare()
{}

/* ---- class Particles -------------------------------------------- */

Particles::Particles() : 
	Tag(),
	particles_axis(),
	particles_script()
{
	particles_entity = false;
	particles_entity_second = false;
	particles_engine = false;
	particles_has_color = false;
	particles_scale = 1.0f;
	particles_cull = CullNone;
}

Particles::Particles(const Particles & other) : 
	Tag(other), 
	particles_axis(other.axis()),
	particles_script(other.script())
{
	particles_entity = other.entity();
	particles_entity_second = other.entity_second();
	particles_engine = other.engine();
	particles_scale = other.scale();
	particles_cull = other.cull();
	particles_has_color = other.has_color();
}

Particles::~Particles()
{
}

/* ---- class Dock ------------------------------------------------- */

Dock::Dock() : Tag()
{
	dock_radius = 0.01f;
}

Dock::Dock(const Dock& other) : Tag(other)
{
	dock_radius = other.radius();
	dock_axis.assign(other.axis());
}

Dock::~Dock()
{
}

/* ---- class Sound ------------------------------------------------- */

Sound::Sound() : Tag()
{
}

Sound::Sound(const Sound& other) : Tag(other),
	sound_name(other.name())
{
}

Sound::~Sound()
{
}

/* ---- class Dock ------------------------------------------------- */

Weapon::Weapon() : Tag()
{
	weapon_radius = 0.01f;
}

Weapon::Weapon(const Weapon& other) : Tag(other)
{
	weapon_radius = other.radius();
	weapon_axis.assign(other.axis());
}

Weapon::~Weapon()
{
}

/* ---- class SubModel---------------------------------------------- */

SubModel::SubModel() : Tag()
{
	submodel_scale = 1.0f;
}

SubModel::SubModel(const SubModel& other) : Tag(other),
		submodel_name(other.name()),
		submodel_axis(other.axis())
{
	submodel_scale = other.scale();
}

SubModel::~SubModel()
{
}

}
